#include "Process.h"
#include <csignal>
#include <unistd.h>
#include <vector>


std::vector<Process*> processes;

void signalHandler(int sig) {
	std::cout << "received signal: " << sig << std::endl;
	for (const auto& process : processes) {
		process->kill();
	}
	exit(1);
}

Process::Process(const std::string& execPath, const std::vector<const char*>& args) : execPath(execPath) {
	this->args.push_back(this->execPath.c_str());
	for (auto& arg : args) {
		this->args.push_back(arg);
	}
	this->args.push_back(0);
	static bool signalHandlerCreated = false;
	if (!signalHandlerCreated) {
		signalHandlerCreated = true;
		signal(SIGTERM, signalHandler);
	}
	processes.push_back(this);
}

Process::~Process() { processes.erase(std::ranges::find(processes, this)); }

void Process::exec() {
	if (pipe(fds) == -1) {
		throw std::runtime_error("Can't create pipe");
	}
	pid = fork();
	if (pid == 0) {
		dup2(fds[1], STDOUT_FILENO);
		close(fds[0]);
		close(fds[1]);
		//execv("/bin/swaybg", "/bin/swaybg" , "-o", output.c_str(), "-i", wallpaperPath.c_str(), "-m", "fill", (char*)0) ;
		execvp(execPath.c_str(), const_cast<char* const*>(args.data())) ;
	} else if (pid > 0) {
		killed = false;
		close(fds[1]);
		//int nbytes = read(fds[0], buf, sizeof(buf));
		//printf("Output: (%.*s)\n", nbytes, buf);
	} else {
		throw std::runtime_error("Can't fork");
	}
}

void Process::kill() {
	if (killed) {
		return;
	}
	killed = true;
	::kill(pid, SIGKILL);
	wait();
}

int Process::wait() {
	int status;
	::waitpid(pid, &status, 0);
	return status;
}
